// CQGCELEventHandlers.cpp : implementation file
//

#include "stdafx.h"
#include "Common.h"
#include "Helper.h"
#include "TradingSystem.h"
#include "TradingSystemDlg.h"

STDMETHODIMP CTradingSystemDlg::OnDataError(
                              LPDISPATCH obj,
                              BSTR errorDescription)
{
   UNREFERENCED_PARAMETER(obj);

   CString errMsg("Data error: ");
   errMsg += ATL::CW2A(errorDescription);
   AfxMessageBox(errMsg);

   return S_OK;
}

STDMETHODIMP CTradingSystemDlg::OnDataConnectionStatusChanged(
                              eConnectionStatus newStatus)
{
   try
   {
      m_lblDataConnection.SetWindowText(_T("Data ") + CHelper::ConnectionStatusToString(newStatus));

      EnableControls();
   }
   catch (std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }

   return S_OK;
}


STDMETHODIMP CTradingSystemDlg::OnTradingSystemDefinitionsResolved(
                              ICQGTradingSystemDefinitions* cqg_tradingSystem_definitions, 
                              ICQGError* cqg_error)
{
   try
   {
      // Raise error, when any error occurred during request
      VARIANT_BOOL isValid;
      HRESULT hr = m_spCQGCEL->IsValid(ATL::CComVariant(cqg_error), &isValid);
      Common::ThrowOnCOMError(m_spCQGCEL, hr);

      if (isValid)
      {
         ATL::CComBSTR errDescription;
         hr = cqg_error->get_Description(&errDescription);
         Common::ThrowOnCOMError<ICQGError>(cqg_error, hr);

         AfxMessageBox(ATL::CW2T(errDescription), MB_ICONEXCLAMATION);

         return S_OK;
      }

      m_spTradingSystemDefinitions.Release();
      m_spTradingSystemDefinitions = cqg_tradingSystem_definitions;

      ShowDefinitions();
   }
   catch (std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }

   return S_OK;
}

STDMETHODIMP CTradingSystemDlg::OnTradingSystemResolved(
                              ICQGTradingSystem* cqg_trading_system,
                              ICQGError* cqg_error)
{
   UNREFERENCED_PARAMETER(cqg_error);

   try
   {
      // Check if trading system resolved for the selected request
      if (IsTradingSystemSelected(cqg_trading_system))
      {
         ClearAllData();
         ShowAllData();

         EnableControls();
      }
   }
   catch (std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }

   return S_OK;
}

STDMETHODIMP CTradingSystemDlg::OnTradingSystemAddNotification(
                              ICQGTradingSystem* cqg_trading_system,
                              ICQGTradingSystemAddInfo* cqg_trading_system_add_info)
{
   UNREFERENCED_PARAMETER(cqg_trading_system_add_info);

   try
   {
      // Check if trading system added for the selected request
      if (IsTradingSystemSelected(cqg_trading_system))
      {
         ShowLastStatistics();
         ShowLastTrades();
      }
   }
   catch (std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }

   return S_OK;
}

STDMETHODIMP CTradingSystemDlg::OnTradingSystemUpdateNotification(
                              ICQGTradingSystem* cqg_trading_system,
                              ICQGTradingSystemUpdateInfo* cqg_trading_system_update_info)
{
   try
   {
      long index;
      HRESULT hr = cqg_trading_system_update_info->get_Index(&index);
      Common::ThrowOnCOMError<ICQGTradingSystemUpdateInfo>(cqg_trading_system_update_info, hr);
   
      // Check if trading system updated for the selected request
      if (IsTradingSystemSelected(cqg_trading_system))
      {
         // Get trades rows
         ATL::CComPtr<ICQGTradingSystemTradesRows> spTradesRows;
         hr = m_spTradingSystem->get_TradesRows(&spTradesRows);
         Common::ThrowOnCOMError(m_spTradingSystem, hr); 

         // Get trades rows count
         long count;
         hr = spTradesRows->get_Count(&count);
         Common::ThrowOnCOMError(spTradesRows, hr);

         const long rowIndex = SHOW_COUNT + index - count;
         if (rowIndex >= 0)
         {
            // Show updated trade in the appropriate row
            ShowTradeRecord(index + 1, m_lvLastTrades, rowIndex);

            // Show updated statistics in the appropriate row
            ShowStatisticRecord(index + 1, m_lvLastStatistics, rowIndex);
         }
      }
   }
   catch (std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }

   return S_OK;
}

STDMETHODIMP CTradingSystemDlg::OnTradingSystemInsertNotification(
                              ICQGTradingSystem* cqg_trading_system,
                              ICQGTradingSystemInsertInfo* cqg_trading_system_insert_info)
{
   UNREFERENCED_PARAMETER(cqg_trading_system_insert_info);

   try
   {
      // Check if trading system inserted for the selected request
      if (IsTradingSystemSelected(cqg_trading_system))
      {
         ShowLastStatistics();
         ShowLastTrades();
      }
   }
   catch (std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }

   return S_OK;
}

STDMETHODIMP CTradingSystemDlg::OnTradingSystemRemoveNotification(
                              ICQGTradingSystem* cqg_trading_system,
                              ICQGTradingSystemRemoveInfo* cqg_trading_system_remove_info)
{
   UNREFERENCED_PARAMETER(cqg_trading_system_remove_info);

   try
   {
      // Check if trading system removed for the selected request
      if (IsTradingSystemSelected(cqg_trading_system))
      {
         ShowLastStatistics();
         ShowLastTrades();
      }
   }
   catch (std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }

   return S_OK;
}