// NumEdit.cpp : implementation file
//

#include "stdafx.h"
#include <sstream>
#include <math.h>
#include "NumEdit.h"

const TCHAR CNumEdit::NEGATION_SYMBOL = _T('-');
const TCHAR CNumEdit::FRACTIONAL_PART_SYMBOL = _T('.');
const COLORREF CNumEdit::NEGATIVE_COLOR = RGB(150, 50, 50);
const COLORREF CNumEdit::POSITIVE_COLOR = RGB(0, 0, 0);

BEGIN_MESSAGE_MAP(CNumEdit, CEdit)
   ON_MESSAGE(WM_PASTE, OnPaste)
   ON_WM_CTLCOLOR_REFLECT()
   ON_WM_RBUTTONDOWN()
   ON_WM_MOUSEWHEEL()
   ON_WM_CHAR()
END_MESSAGE_MAP()

CNumEdit::CNumEdit()
{
   m_allowNegativeValues = false;
   m_allowFractionalPart = false;
   m_isNegative = false;

   m_maxIntegerPartLength = 7;
   m_maxFractionalPartLength = 4;
}

BOOL CNumEdit::OnMouseWheel(
                        UINT nFlags,
                        short zDelta,
                        CPoint pt)
{
   UNREFERENCED_PARAMETER(nFlags);
   UNREFERENCED_PARAMETER(pt);

   if (m_allowFractionalPart)
   {
      const double delta = zDelta / WHEEL_DELTA;
      IncreaseValue(delta);
   }
   else
   {
      const int delta = zDelta / WHEEL_DELTA;
      IncreaseValue(delta);
   }
   return TRUE;
}

afx_msg void CNumEdit::OnRButtonDown(
                        UINT nFlags,
                        CPoint point)
{
   UNREFERENCED_PARAMETER(nFlags);
   UNREFERENCED_PARAMETER(point);

   return;
}

LRESULT CNumEdit::OnPaste(
                        WPARAM wParam,
                        LPARAM lParam)
{
   UNREFERENCED_PARAMETER(wParam);
   UNREFERENCED_PARAMETER(lParam);

   return 0L;
}

HBRUSH CNumEdit::CtlColor(
                        CDC* pDC,
                        UINT nCtlColor) 
{
   UNREFERENCED_PARAMETER(nCtlColor);

   COLORREF textColor = (m_isNegative ? NEGATIVE_COLOR : POSITIVE_COLOR);

   // Set text color
   pDC->SetTextColor(textColor);
   pDC->SetBkMode(TRANSPARENT);

   // Return the brush
   return (HBRUSH)GetStockObject(DEFAULT_PALETTE);
}

BOOL CNumEdit::PreTranslateMessage(
                        MSG *pMSG)
{
	if ((WM_KEYDOWN == pMSG->message) || (WM_KEYUP == pMSG->message))
   {
      if ((WM_KEYDOWN == pMSG->message) && (VK_UP == pMSG->wParam))
      {
         // Handle key up, increase value by 1
         if (m_allowFractionalPart)
         {
            IncreaseValue(1.0);
         }
         else
         {
            IncreaseValue(1);
         }
         return TRUE;
      }
      else if ((WM_KEYDOWN == pMSG->message) && (VK_DOWN == pMSG->wParam))
      {
         // Handle key down, decrease value by 1
         if (m_allowFractionalPart)
         {
            DecreaseValue(1.0);
         }
         else
         {
            DecreaseValue(1);
         }
         return TRUE;
      }
      else if ((WM_KEYDOWN == pMSG->message) && (VK_DELETE == pMSG->wParam))
      {
         if (!CheckDelete())
         {
            return TRUE;
         }
      }

      // Get edit box text
      CString valueText;
      GetWindowText(valueText);

      // Validate if new text is different from last set text
      if (m_lastText != valueText)
      {
         long value = 0;
         double dblValue = 0;

         if (valueText.IsEmpty() || NEGATION_SYMBOL == valueText || FRACTIONAL_PART_SYMBOL == valueText)
         {
            // If new text is empty or contain only negation symbol then set value to 0
            m_isNegative = false;
         }
         else
         {
            // Remove first character if it is a negation symbol
            m_isNegative = (NEGATION_SYMBOL == valueText[0]);
            if (m_isNegative)
            {
               valueText.Delete(0);
            }

            // Convert to long or double and multiple by -1 if negative
            if (m_allowFractionalPart)
            {
               dblValue = (m_isNegative ? -1 : 1) * _tstof(valueText);
            }
            else
            {
               value = (m_isNegative ? -1 : 1) * _tstoi(valueText);
            }
         }

         // Set validated text into edit control
         if (m_allowFractionalPart)
         {
            SetEditBoxDblValue(dblValue);
         }
         else
         {
            SetEditBoxValue(value);
         }
      }
   }

   return CEdit::PreTranslateMessage(pMSG);
}

void CNumEdit::OnChar(
                        UINT nChar,
                        UINT nRepCnt,
                        UINT nFlags) 
{
   if (NEGATION_SYMBOL == nChar)
   {
      // Check if negative values are allowed
      if (m_allowNegativeValues)
      {
         if (m_allowFractionalPart)
         {
            double value = GetEditBoxDblValue();

            SetEditBoxDblValue(-1 * value);
            return;
         }
         else
         {
            long value = GetEditBoxValue();

            SetEditBoxValue(-1 * value);
            return;
         }
      }
   }
   else if (FRACTIONAL_PART_SYMBOL == nChar)
   {
      // Check if fractional part allowed
      if (m_allowFractionalPart)
      {
         // Get edit box text
         CString valueText;
         GetWindowText(valueText);

         if (-1 == valueText.Find(FRACTIONAL_PART_SYMBOL))
         {
            CEdit::OnChar(nChar, nRepCnt, nFlags);
            return;
         }
      }
   }
   else if (isdigit(nChar)) // '0' <= nChar <= '9'
   {
      if (CheckDigit())
      {
         CEdit::OnChar(nChar, nRepCnt, nFlags);
         return;
      }
   }
   else if (VK_BACK == nChar)
   {
      if (CheckBack())
      {
         CEdit::OnChar(nChar, nRepCnt, nFlags);
         return;
      }
   }

   ::MessageBeep(MB_ICONEXCLAMATION);
}

bool CNumEdit::CheckDigit() const
{
   // Get edit box text
   CString valueText;
   GetWindowText(valueText);

   // Get selection range
   int selStart;
   int selEnd;
   GetSel(selStart, selEnd);

   // When maximum number of symbols was exceeded do nothing
   if (!valueText.IsEmpty())
   {
      // Remove first character if it is a negation symbol
      if (m_isNegative)
      {
         valueText.Delete(0);
         selStart--;
         selEnd--;
      }

      // Check maximum number of symbols
      if (!m_allowFractionalPart)
      {
         if ((valueText.GetLength() >= m_maxIntegerPartLength) && (selStart == selEnd))
         {
            return false;
         }
      }
      else
      {
         // Find fractional part symbol position
         const int fpPos = valueText.Find(FRACTIONAL_PART_SYMBOL);

         // Check maximum number of symbols in integer part
         if (-1 == fpPos)
         {
            if ((valueText.GetLength() >= m_maxIntegerPartLength) && (selStart == selEnd))
            {
               return false;
            }
         }
         else
         {
            // If fractional part symbol is selected do nothing
            if (selStart <= fpPos && selEnd > fpPos)
            {
               return false;
            }

            if ((fpPos >= m_maxIntegerPartLength) && 
               (selStart <= fpPos) && 
               (selStart == selEnd))
            {
               return false;
            }

            // Check maximum number of symbols in fractional part
            if (((valueText.GetLength() - fpPos) > m_maxFractionalPartLength) && 
               (selStart > fpPos) && 
               (selStart == selEnd))
            {
               return false;
            }
         }
      }
   }

   // When negation symbol exist and cursor is in the first position do nothing
   if (m_isNegative && !selStart && !selEnd)
   {
      return false;
   }

   return true;
}

bool CNumEdit::CheckBack() const
{
   if (m_allowFractionalPart)
   {
      // Get edit box text
      CString valueText;
      GetWindowText(valueText);

      // Get selection range
      int selStart;
      int selEnd;
      GetSel(selStart, selEnd);

      // Find fractional part symbol position
      const int fpPos = valueText.Find(FRACTIONAL_PART_SYMBOL);

      if (-1 != fpPos)
      {
         // If fractional part symbol is selected do nothing
         if (selStart <= fpPos && selEnd > fpPos)
         {
            return false;
         }

         // If fractional part symbol will be removed do nothing
         if (selStart == selEnd && selStart == fpPos + 1)
         {
            return false;
         }
      }
   }

   return true;
}

bool CNumEdit::CheckDelete() const
{
   // Get edit box text
   CString valueText;
   GetWindowText(valueText);

   // Get selection range
   int selStart;
   int selEnd;
   GetSel(selStart, selEnd);

   if (m_allowFractionalPart)
   {
      // Find fractional part symbol position
      const int fpPos = valueText.Find(FRACTIONAL_PART_SYMBOL);

      if (-1 != fpPos)
      {
         // If fractional part symbol is selected do nothing
         if (selStart <= fpPos && selEnd > fpPos)
         {
            return false;
         }

         // If fractional part symbol will be removed do nothing
         if (selStart == selEnd && selStart == fpPos)
         {
            return false;
         }
      }
   }

   return true;
}

bool CNumEdit::CheckValue(
                        long value) const
{
   value = abs(value);

   CString valueText;
   valueText.Format(_T("%d"), value);

   // Check maximum number of symbols
   if (valueText.GetLength() > m_maxIntegerPartLength)
   {
      return false;
   }

   return true;
}

bool CNumEdit::CheckDblValue(
                        double value) const
{
   value = fabs(value);

   CString valueText;
   valueText.Format(_T("%f"), value);

   // Check maximum number of symbols
   if (!m_allowFractionalPart)
   {
      if (valueText.GetLength() > m_maxIntegerPartLength)
      {
         return false;
      }
   }
   else
   {
      // Find fractional part symbol position
      const int fpPos = valueText.Find(FRACTIONAL_PART_SYMBOL);

      if (-1 == fpPos)
      {
         if (valueText.GetLength() > m_maxIntegerPartLength)
         {
            return false;
         }
      }
      else
      {
         // Check maximum number of symbols in integer part
         if (fpPos > m_maxIntegerPartLength)
         {
            return false;
         }
      }
   }

   return true;
}

long CNumEdit::GetEditBoxValue() const
{
   // Get edit box text
   CString valueText;
   GetWindowText(valueText);

   // Return 0 when text is empty
   if (valueText.IsEmpty())
   {
      return 0;
   }

   // Remove negation sign if exist
   if (m_isNegative)
   {
      valueText.Delete(0);
   }

   // Convert to long and multiple by -1 if value is negative
   return (m_isNegative ? -1 : 1) * _tstoi(valueText);
}

double CNumEdit::GetEditBoxDblValue() const
{
   // Get edit box text
   CString valueText;
   GetWindowText(valueText);

   // Return 0 when text is empty
   if (valueText.IsEmpty())
   {
      return 0.0;
   }

   // Remove negation sign if exist
   if (m_isNegative)
   {
      valueText.Delete(0);
   }

   // Convert to double and multiple by -1 if value is negative
   return (m_isNegative ? -1 : 1) * _tstof(valueText);
}

void CNumEdit::SetEditBoxValue(
                        long value)
{
   // Get selection range
   int selStart;
   int selEnd;
   GetSel(selStart, selEnd);

   // Get negation state
   m_isNegative = (m_allowNegativeValues && (value < 0));
   value = abs(value);

   // Convert value to string and add negation sign if needed
   CString valueText;
   valueText.Format(_T("%d"), value);
   if (m_isNegative)
   {
      valueText = NEGATION_SYMBOL + valueText;
   }

   // Set edit box text
   SetWindowText(valueText);

   m_lastText = valueText;

   // Set selection range
   SetSel(selStart, selEnd);
}

void CNumEdit::SetEditBoxDblValue(
                        double value)
{
   // Get selection range
   int selStart;
   int selEnd;
   GetSel(selStart, selEnd);

   // Get negation state
   m_isNegative = (m_allowNegativeValues && (value < 0));
   value = fabs(value);

   // Convert value to string and add negation sign if needed
   CString formatStr;
   formatStr.AppendFormat(_T("%%.%df"), m_maxFractionalPartLength);

   CString valueText;
   valueText.Format(formatStr, value);
   
   if (m_isNegative)
   {
      valueText = NEGATION_SYMBOL + valueText;
   }

   // Removing zeros in fractional part
   if (0 < valueText.Find(FRACTIONAL_PART_SYMBOL))
   {
      valueText.TrimRight('0');
   }

   // Set edit box text
   SetWindowText(valueText);

   m_lastText = valueText;

   // Set selection range
   SetSel(selStart, selEnd);
}

void  CNumEdit::SetNegativeAllowed(
                        bool isNegativeAllowed)
{
   m_allowNegativeValues = isNegativeAllowed;
}

void  CNumEdit::SetFractionAllowed(
                        bool isFractionAllowed)
{
   m_allowFractionalPart = isFractionAllowed;
}

void  CNumEdit::SetMaxIntegerPartLength(
                        int maxIntegerPartLength)
{
   m_maxIntegerPartLength = maxIntegerPartLength;
}

void  CNumEdit::SetMaxFractionalPartLength(
                        int maxFractionalPartLength)
{
   m_maxFractionalPartLength = maxFractionalPartLength;
}

void CNumEdit::SetValue(
                        long value)
{
   SetEditBoxValue(value);
}

void CNumEdit::SetVariantValue(
                        const ATL::CComVariant& value)
{
   if (VT_R8 == value.vt)
   {
      m_allowFractionalPart = true;
      SetEditBoxDblValue(value.dblVal);
   }
   else
   {
      m_allowFractionalPart = false;
      SetEditBoxValue(value.lVal);
   }
}

long CNumEdit::GetValue() const
{
   return GetEditBoxValue();
}

ATL::CComVariant CNumEdit::GetVariantValue() const
{
   ATL::CComVariant value;

   if (m_allowFractionalPart)
   {
      value = ATL::CComVariant(GetEditBoxDblValue(), VT_R8);
   }
   else
   {
      value = ATL::CComVariant(GetEditBoxValue(), VT_I4);
   }

   return value;
}

void CNumEdit::IncreaseValue(
                        int delta)
{
   long value = GetEditBoxValue();
   value += delta;
   if (!m_allowNegativeValues && value < 0)
   {
      value = 0;
   }

   if (CheckValue(value))
   {
      SetEditBoxValue(value);
   }
}

void CNumEdit::IncreaseValue(
                        double delta)
{
   if (!m_allowFractionalPart)
   {
      return;
   }

   double value = GetEditBoxDblValue();
   value += delta;
   if (!m_allowNegativeValues && value < 0)
   {
      value = 0;
   }

   if (CheckDblValue(value))
   {
      SetEditBoxDblValue(value);
   }
}

void CNumEdit::DecreaseValue(
                        int delta)
{
   long value = GetEditBoxValue();
   value -= delta;
   if (!m_allowNegativeValues && value < 0)
   {
      value = 0;
   }

   if (CheckValue(value))
   {
      SetEditBoxValue(value);
   }
}

void CNumEdit::DecreaseValue(
                        double delta)
{
   double value = GetEditBoxDblValue();
   value -= delta;
   if (!m_allowNegativeValues && value < 0)
   {
      value = 0;
   }

   if (CheckDblValue(value))
   {
      SetEditBoxDblValue(value);
   }
}
